//--------------------------------------------------------------------------
//
// Fres: Free Resources Display Application
// Displays: Date, Time, Windows Mode, Global free memory,
// limiting resource heap, and percentage of free resource heap.
//
// Demonstrates the use of a non-modal dialog as the main window,
// the use of the undocumented function: GetHeapSpaces (Win 3.0),
// and the use of GetFreeSystemsResources (Win 3.1).
//
// Design & Coding March 1992 by Jamie O'Connell
// Copyright (c) 1992 by Jamie O'Connell
//
//--------------------------------------------------------------------------

#include "windows.h"
#include <time.h>
#include <string.h>

#define  VERSION	"1.52"
#define  IDMEM	    102
#define  IDFSR	    103
#define  IDVERSTR   113

#define  GDIHP	 1
#define  USERHP  2

#define  IDM_ABOUT  0x10
#define  IDM_ONTOP  0x11
#define  IDM_SAVLOC 0x12

//---------------------------------------------------------------------------
// Global Variables...
//---------------------------------------------------------------------------

HANDLE  hInstance;              // Global instance handle for application
int     hwndKill, idKill;
int 	Millisec = 800; 		// Current timer speed in milliseconds
HWND    hwndMain;               // Main hwnd.  Needed in callback
int     counter = 0;            // Counter to display when timer goes off
char	szDT[32];				// storage to build strings
char	szMem[32];				// storage to build strings
char	szRes[32];				// storage to build strings
char    szAppName[]  = "Fres";      // Name of App
char    szOnTopKey[] = "StayOnTop"; // Profile key
char    szVerStr[12];

BOOL    fOnTop = FALSE;
HMENU   hSysMenu;
RECT    rPos;
FARPROC lpfnAbtDlg;
FARPROC lpfnDlgProc;
WORD   (*fnHeapInfo)(VOID);

//---------------------------------------------------------------------------
// Function declarations
//---------------------------------------------------------------------------

BOOL FAR PASCAL DlgProc(HWND hwnd, WORD msg, WORD wParam, LONG lParam);
BOOL FAR PASCAL AbtDlg(HWND hwnd, WORD msg, WORD wParam, LONG lParam);
BOOL FormatDT(char *szStr);
BOOL FormatMem(char *szStr);
BOOL FormatSR(char *szStr);
WORD HeapInfo(VOID);
WORD Hp31Info(VOID);

//---------------------------------------------------------------------------
// WinMain
//---------------------------------------------------------------------------

int PASCAL WinMain(HANDLE hInst, HANDLE hInstPrev,
				   LPSTR lpstrCmdLine, int cmdShow) {
	MSG  msgMain;
	char tmp[6];
	WORD Vers;
        
	// Set the global instance variable
	hInstance = hInst;

	wsprintf((LPSTR)szVerStr, "%s v%s", (LPSTR)szAppName, (LPSTR)VERSION);

	Vers = GetVersion();
	if ((LOBYTE(Vers) == 3) && (HIBYTE(Vers) == 0))
		fnHeapInfo = HeapInfo; // Win 3.0
	else
		fnHeapInfo = Hp31Info; // Win 3.1+ Use new method

	lpfnDlgProc = MakeProcInstance(DlgProc, hInst);
	if ((hwndMain = CreateDialog(hInst, szAppName,
								 NULL, lpfnDlgProc)) == NULL)
		return(0);

	// Add About Item to Sytem menu
	hSysMenu = GetSystemMenu(hwndMain, FALSE);
	AppendMenu(hSysMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(hSysMenu, MF_STRING, IDM_ABOUT, 	"&About...");
	AppendMenu(hSysMenu, MF_STRING, IDM_ONTOP,   "&Stay on Top");
	AppendMenu(hSysMenu, MF_STRING, IDM_SAVLOC,  "&Save Location");

	GetProfileString(szAppName, szOnTopKey, "off", tmp, sizeof(tmp));
	if (stricmp(tmp, "on")   == 0 ||
	    stricmp(tmp, "yes")  == 0 ||
	    stricmp(tmp, "true") == 0 ||
	    stricmp(tmp, "1")	 == 0) {
		fOnTop = TRUE;
		CheckMenuItem(hSysMenu, IDM_ONTOP, MF_CHECKED);
		}

	// Main message "pump"
	while (GetMessage((LPMSG) &msgMain, NULL, 0, 0)) {
		if (!IsDialogMessage(hwndMain, (LPMSG)&msgMain)) {
			TranslateMessage((LPMSG) &msgMain);
			DispatchMessage((LPMSG) &msgMain);
			}
		}

	FreeProcInstance(lpfnDlgProc);
	return(0);
	}


//===========================================================================
// AbtDlg
//
// Dialog Callback
//
//===========================================================================

BOOL FAR PASCAL AbtDlg(HWND hwnd, WORD msg, WORD wParam, LONG lParam) {

	switch(msg) {
		case WM_INITDIALOG:
			SetDlgItemText(hwnd, IDVERSTR, szVerStr);
			return(TRUE);

		case WM_CLOSE:
			EndDialog(hwnd, 0);
			return(TRUE);

		case WM_COMMAND:
			switch(wParam) {
				case IDOK:
					EndDialog(hwnd, 0);
					return(TRUE);
				}
			break;
		}
	return(FALSE);
	}

//===========================================================================
// WndProc
//
// Window procedure for the sample applications window.
//
//===========================================================================
BOOL FAR PASCAL DlgProc(HWND hwnd, WORD msg, WORD wParam, LONG lParam) {
	int xpos, ypos, nWidth, nHeight;
	WORD wCheck;
	char tmp[6];

	switch(msg) {
		case WM_INITDIALOG:
			GetWindowRect(hwnd, (LPRECT)&rPos);
			nWidth = rPos.right - rPos.left;
			nHeight = rPos.bottom - rPos.top;
			xpos = GetProfileInt(szAppName, "X", rPos.left);
			ypos = GetProfileInt(szAppName, "Y", rPos.top);
			MoveWindow(hwnd, xpos, ypos, nWidth, nHeight, TRUE);

			lpfnAbtDlg  = MakeProcInstance(AbtDlg, hInstance);
			if (SetTimer(hwnd, 1, Millisec, NULL) == NULL) {
				MessageBox(hwnd, "Couldn't create timer.", "Notice!", MB_OK);
				DestroyWindow(hwnd);
				PostQuitMessage(0);
				return(TRUE);
				}
			hwndKill = hwnd;
			idKill = 1;

		case WM_TIMER:
			if (fOnTop & GetWindow(hwnd, GW_HWNDPREV) != hwnd)
				SetWindowPos(hwnd, NULL, 0, 0, 0, 0,
					  SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
			if (FormatDT(szDT)) {
				SetWindowText(hwnd, szDT); // Time has changed
				PostMessage(hwnd, WM_NCPAINT, 0, 0L); // redo border
				}
			if (FormatMem(szMem))
				SetDlgItemText(hwnd, IDMEM, szMem);
			if (FormatSR(szRes))
				SetDlgItemText(hwnd, IDFSR, szRes);

			return(TRUE);

		case WM_CLOSE:
			KillTimer(hwndKill, idKill);
			idKill = 0;

			FreeProcInstance(lpfnAbtDlg);
			DestroyWindow(hwnd);
			PostQuitMessage(0);
			return(TRUE);

		case WM_SYSCOMMAND:
			switch(wParam) {
				case IDM_ABOUT:
					DialogBox(hInstance, "About", hwnd, lpfnAbtDlg);
					return(TRUE);

				case IDM_ONTOP:
					if (fOnTop) {
						fOnTop = FALSE;
						wCheck = MF_UNCHECKED;
						strcpy(tmp, "off");
						}
					else {
						fOnTop = TRUE;
						wCheck = MF_CHECKED;
						strcpy(tmp, "on");
						}
					CheckMenuItem(hSysMenu, IDM_ONTOP, wCheck);
					WriteProfileString(szAppName, szOnTopKey, tmp);
					return(TRUE);

				case IDM_SAVLOC:
					GetWindowRect(hwndMain, (LPRECT)&rPos);
					wsprintf((LPSTR)tmp, "%d", rPos.left);
					WriteProfileString(szAppName, "X", tmp);
					wsprintf((LPSTR)tmp, "%d", rPos.top);
					WriteProfileString(szAppName, "Y", tmp);
					return(TRUE);

				}
			break;

		}
	return(FALSE);
	}

//---------------------------------------------------------------------------
// FormatDT
//
// Formats The Date and time - Returns TRUE if it has changed.
//---------------------------------------------------------------------------

BOOL FormatDT(char *szStr) {
	static short savehr = -1;
	static short savemin = -1;
	struct tm *tym;
	time_t timer = 0;
	short  hour;
	short  mini;
	char   suffix[5];

	timer = time(&timer);
	tym = localtime(&timer);

	hour = tym->tm_hour;
	mini = tym->tm_min;

	if (hour == savehr && mini == savemin) // hasn't changed enough
	  return FALSE;

	savehr  = hour;
	savemin = mini;

	if (hour > 12) {
		hour = hour - 12;
		strcpy(suffix, "pm");
		}
	else if (hour == 12) {
		if (mini == 0)
			strcpy(suffix, "noon");
		else
			strcpy(suffix, "pm");
		}
	else {
		strcpy(suffix, "am");
		if (hour == 0) {
			hour = 12;
			if (mini == 0)
				strcpy(suffix, "nite");
			}
		}
	wsprintf((LPSTR)szStr, "%2.2d-%2.2d-%2.2d %2d:%2.2d %s",
			  tym->tm_mon+1, tym->tm_mday, tym->tm_year,
			  hour, mini, (LPSTR)suffix);
	return TRUE;
	}

//---------------------------------------------------------------------------
// FormatMem
//
// Formats the free memory string - Returns TRUE if it's changed.
//---------------------------------------------------------------------------

BOOL FormatMem(char *szStr) {
	static WORD savemem = 0;
	WORD mem;

	mem = (WORD)(GetFreeSpace(0) >> 10); // get #of KBytes
	if (mem == savemem)
		return FALSE;

	savemem = mem;

	wsprintf((LPSTR)szStr, "%s: %5u K",
             (LPSTR)((GetWinFlags() & WF_ENHANCED) ? "Enh" : "Std"), mem);

	return TRUE;
	}

//---------------------------------------------------------------------------
// FormatSR
//
// Formats the free system resources string - Returns TRUE if it's changed.
//---------------------------------------------------------------------------

BOOL FormatSR(char *szStr) {
	static WORD wResav = 0;
	short nres;
	short nmod;
	WORD  wResult;

	wResult = fnHeapInfo(); // Function Variable
	if (wResult == wResav)	// Unchanged
		return FALSE;

	wResav = wResult;
	nres   = HIBYTE(wResult);
	nmod   = LOBYTE(wResult);

	wsprintf((LPSTR)szStr, "%s: %2.2d%%",
		   (LPSTR)((nmod == GDIHP) ? "GDI" : "User"), nres);

	return TRUE;
	}

//---------------------------------------------------------------------------
// HeapInfo
//
// Determines the sizes and percentages of the 2 resource heaps - used
// only if Windows 3.0 is running
//---------------------------------------------------------------------------

WORD HeapInfo(VOID) {
	static HMODULE hGDI  = NULL;
	static HMODULE hUser = NULL;

	DWORD FAR PASCAL GetHeapSpaces(HMODULE);  // Undoc call

	BYTE  PctGDI, PctUser;
	DWORD dwInfo;
	DWORD dwFree, dwSize;
	WORD  RetVal;

	// Initialize static
	if (hUser == NULL || hGDI == NULL) {
		hUser = GetModuleHandle("User");
		hGDI  = GetModuleHandle("GDI");
		}

	/// Info for User Heap
	dwInfo	= GetHeapSpaces(hUser);
	dwSize	= (DWORD)HIWORD(dwInfo);
	dwFree	= (DWORD)LOWORD(dwInfo);
	PctUser =  (BYTE)((dwFree * 100) / dwSize);

	// Info for GDI
	dwInfo = GetHeapSpaces(hGDI);
	dwSize = (DWORD)HIWORD(dwInfo);
	dwFree = (DWORD)LOWORD(dwInfo);
	PctGDI = (BYTE)((dwFree * 100) / dwSize);

	if (PctUser < PctGDI)
		RetVal = (WORD)((PctUser << 8) | USERHP);
	else
		RetVal = (WORD)((PctGDI  << 8) | GDIHP);

	return RetVal;
	}

//---------------------------------------------------------------------------
// Hp31Info
//
// Determines the sizes and percentages of the 2 resource heaps - used
// only if Windows 3.1+ is running
//---------------------------------------------------------------------------

WORD Hp31Info(VOID) {
	BYTE  PctGDI, PctUser;
	WORD  RetVal;

	PctGDI	= GetFreeSystemResources(GDIHP);
	PctUser = GetFreeSystemResources(USERHP);

	if (PctUser < PctGDI)
		RetVal = (WORD)((PctUser << 8) | USERHP);
	else
		RetVal = (WORD)((PctGDI  << 8) | GDIHP);

	return RetVal;
	}
